﻿Type=Class
Version=6.8
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'	CameraManager class

#Event: CameraConnected(Success As Boolean)
#Event: GPSConnected(Success As Boolean)

Sub Class_Globals
	Private GPS_INFO_UPDATE_INTERVAL As Int=15000
	Private KEY_CODE_DEBUG As Int=88	'	joystick UP
	Private KEY_CODE_NONE As Int=-1
	Private KEY_CODE_TAKE_PICTURE As Int=85
	
	Private mCameraIsConnected As Boolean=False
	Private mEventName As String
	Private mFusedLocationProvider1 As FusedLocationProvider
	Private mGpsInfoTimer As Timer
	Private mLastLocation As Location
	Private mLastKeyCode As Int=KEY_CODE_NONE
	Private mMediaSession1 As MediaSession
	Private mOpenSphericalCamera1 As OSC_OpenSphericalCamera
	Private mParent As Object
End Sub

public Sub CameraConnect
	mOpenSphericalCamera1.StartSession(GetCameraAddress)
End Sub

public Sub CameraDisconnect
	mOpenSphericalCamera1.CloseSession
	CallSubDelayed2(mParent, (mEventName&"_CameraConnected").ToLowerCase, False)
End Sub

Private Sub GetCameraAddress As String
	Dim ServerSocket1 As ServerSocket
	ServerSocket1.Initialize(0, "")
	Dim HostAddress As String=ServerSocket1.GetMyWifiIP
	
	'	Log("HostAddress: "&HostAddress)
	
	Dim HostAddressOctets() As String=Regex.Split("\.", HostAddress)
	Dim CameraAddress As StringBuilder
	CameraAddress.Initialize
	CameraAddress.Append(HostAddressOctets(0)).append(".").append(HostAddressOctets(1)).append(".").append(HostAddressOctets(2)).append(".1")
	'	Log("CameraAddress: "&CameraAddress.ToString)
	Return CameraAddress.ToString
End Sub 

public Sub GPSDisable
	mFusedLocationProvider1.Disconnect
End Sub

public Sub GPSEnable
	mFusedLocationProvider1.Connect
End Sub

Public Sub Initialize(Parent As Object, EventName As String)
	mEventName=EventName
	mParent=Parent
	
	mFusedLocationProvider1.Initialize("mFusedLocationProvider1")
	
	mGpsInfoTimer.Initialize("mGpsInfoTimer", GPS_INFO_UPDATE_INTERVAL)
	
	mMediaSession1.Initialize("mMediaSession1")
	' flags: MediaSession.FLAG_HANDLES_MEDIA_BUTTONS | MediaSession.FLAG_HANDLES_TRANSPORT_CONTROLS
	mMediaSession1.SetFlags(3)
	
	mOpenSphericalCamera1.Initialize("mOpenSphericalCamera1")
End Sub

public Sub IsCameraConnected As Boolean
	Return mCameraIsConnected
End Sub

public Sub IsGPSEnabled As Boolean
	Return mGpsInfoTimer.Enabled
End Sub

public Sub IsRemoteEnabled As Boolean
	Return mMediaSession1.IsActive
End Sub

private Sub mFusedLocationProvider1_ConnectionFailed(ConnectionResult1 As Int)
	Log("mFusedLocationProvider1_ConnectionFailed: "&ConnectionResult1)
End Sub

private Sub mFusedLocationProvider1_ConnectionSuccess
	Log("mFusedLocationProvider1_ConnectionSuccess")
	
	Dim LocationRequest1 As LocationRequest
	LocationRequest1.Initialize
	LocationRequest1.SetInterval(GPS_INFO_UPDATE_INTERVAL/2)	'	TODO
	LocationRequest1.SetPriority(LocationRequest1.Priority.PRIORITY_HIGH_ACCURACY)
	LocationRequest1.SetSmallestDisplacement(1)	'	1 meter
	
	Dim LocationSettingsRequestBuilder1 As LocationSettingsRequestBuilder
	LocationSettingsRequestBuilder1.Initialize
	LocationSettingsRequestBuilder1.AddLocationRequest(LocationRequest1)
	mFusedLocationProvider1.CheckLocationSettings(LocationSettingsRequestBuilder1.Build)
	
	mFusedLocationProvider1.RequestLocationUpdates(LocationRequest1)
End Sub

private Sub mFusedLocationProvider1_ConnectionSuspended(SuspendedCause1 As Int)
	Log("mFusedLocationProvider1_ConnectionSuspended: "&SuspendedCause1)
	Select SuspendedCause1
		Case mFusedLocationProvider1.SuspendedCause.CAUSE_NETWORK_LOST
			'	TODO take action
		Case mFusedLocationProvider1.SuspendedCause.CAUSE_SERVICE_DISCONNECTED
			'	TODO take action
	End Select
	
	mGpsInfoTimer.Enabled=False
	CallSubDelayed2(mParent, (mEventName&"_GPSConnected").ToLowerCase, True)
End Sub

private Sub mFusedLocationProvider1_LocationChanged(Location1 As Location)
	Log("mFusedLocationProvider1_LocationChanged")
	
	
	mLastLocation=Location1
	If Not(mGpsInfoTimer.Enabled) Then
		'	make sure first fix is always sent to camera
		mGpsInfoTimer_Tick
		mGpsInfoTimer.Enabled=True
		CallSubDelayed2(mParent, (mEventName&"_GPSConnected").ToLowerCase, True)
	End If
End Sub

private Sub mFusedLocationProvider1_LocationSettingsChecked(LocationSettingsResult1 As LocationSettingsResult)
	Log("mFusedLocationProvider1_LocationSettingsChecked")
	Dim LocationSettingsStatus1 As LocationSettingsStatus=LocationSettingsResult1.GetLocationSettingsStatus
	Select LocationSettingsStatus1.GetStatusCode
		Case LocationSettingsStatus1.StatusCodes.RESOLUTION_REQUIRED
			Log("RESOLUTION_REQUIRED")
			'	device settings do not meet the location request requirements
			'	a resolution dialog is available to enable the user to change the settings
			'	the  StartResolutionDialog method cannot be called from a service
			'	so we'll pass the LocationSettingsStatus to the Main activity to be handled
			'	CallSubDelayed2(Main, "HandleLocationSettingsStatus", LocationSettingsStatus1)
		Case LocationSettingsStatus1.StatusCodes.SETTINGS_CHANGE_UNAVAILABLE
			Log("SETTINGS_CHANGE_UNAVAILABLE")
			'	device settings do not meet the location request requirements
			'	a resolution dialog is not available to enable the user to change the settings
			'	we'll pass the LocationSettingsStatus to the Main activity to be handled
			'	CallSubDelayed2(Main, "HandleLocationSettingsStatus", LocationSettingsStatus1)
		Case LocationSettingsStatus1.StatusCodes.SUCCESS
			Log("SUCCESS")
			'	device settings meet the location request requirements
			'	no further action required
			
	End Select
End Sub

private Sub mGpsInfoTimer_Tick
	Dim GpsInfoObject As Map
	GpsInfoObject.Initialize
	GpsInfoObject.Put("lat", mLastLocation.Latitude)
	GpsInfoObject.Put("lng", mLastLocation.Longitude)
	GpsInfoObject.Put("_altitude", mLastLocation.Altitude)	'	Theta S only?
	
	Log(GpsInfoObject)	'	shows correct lat and lng properties have been created
	
	Dim GpsInfoProperty As Map
	GpsInfoProperty.Initialize
	GpsInfoProperty.Put("gpsInfo", GpsInfoObject)
	
	Dim GpsInfoOption As JSONGenerator
	GpsInfoOption.Initialize(GpsInfoProperty)
	
	mOpenSphericalCamera1.SetOptions(GpsInfoOption)
	Log("GPS info request made")
End Sub

private Sub mMediaSession1_MediaButton(KeyCode As Int) As Boolean
	Log("KeyCode: "&KeyCode)
	Dim EventHandled As Boolean=False
	If KeyCode<>mLastKeyCode Then
		mLastKeyCode=KeyCode
		Select KeyCode
			Case KEY_CODE_DEBUG
				'	mOpenSphericalCamera1.GetOptions(Array As String("gpsInfo"))	'	"captureModeSupport" works
				
				Dim IsoOptionsMap As Map
				IsoOptionsMap.Initialize
				IsoOptionsMap.Put("iso", 200)
				
				Dim IsoOptionsObject As JSONGenerator
				IsoOptionsObject.Initialize(IsoOptionsMap)
				
				'	mOpenSphericalCamera1.SetOptions(IsoOptionsObject)
				
			Case KEY_CODE_TAKE_PICTURE
				If mCameraIsConnected Then
					mOpenSphericalCamera1.TakePicture
					EventHandled=True
				End If
		End Select
	Else
		Log("Ignoring repeating keycode")
	End If
	Return EventHandled
End Sub

private Sub mOpenSphericalCamera1_OptionsReady(Success As Boolean, Options As JSONParser)
	Log("mOpenSphericalCamera1_OptionsReady: "&Success)
	If Success Then
		Log(Options)
	Else
		If LastException.IsInitialized Then
			'	get gpsinfo request is timing out?
			Log(LastException.Message)
		End If
	End If
End Sub

private Sub mOpenSphericalCamera1_OptionsSet(Success As Boolean, CameraResponseError As JSONParser)
	Log("mOpenSphericalCamera1_OptionsSet: "&Success)
	If Success Then
		ToastMessageShow("options set", False)
	Else
		Log(CameraResponseError)
	End If
End Sub

private Sub mOpenSphericalCamera1_PictureTaken(Success As Boolean, FileUri As String, CameraResponse As JSONParser)
	Log("mOpenSphericalCamera1_PictureTaken: "&Success)
	If mLastKeyCode=KEY_CODE_TAKE_PICTURE Then
		mLastKeyCode=KEY_CODE_NONE
	End If
End Sub

private Sub mOpenSphericalCamera1_SessionClosed(Success As Boolean, CameraResponse As JSONParser)
	Log("mOpenSphericalCamera1_SessionClosed: "&Success)
	mCameraIsConnected=False
End Sub

private Sub mOpenSphericalCamera1_SessionStarted(Success As Boolean, CameraResponse As JSONParser)
	Log("mOpenSphericalCamera1_SessionStarted")
	mCameraIsConnected=Success
	Select Success
		Case False
			Log("Failed to connect")
			
		Case True
			Log("Connected to camera")
			
	End Select
	
'	Dim Callback1 As Callback=mOpenSphericalCamera1.Tag
'	Dim Object1 As Object
'	'mOpenSphericalCamera1.Tag=Object1
'	
'	
'	If Callback1.IsInitialized Then
'		CallSubDelayed2(Callback1.Parent, Callback1.SubName, Success)
'	End If

	CallSubDelayed2(mParent, (mEventName&"_CameraConnected").ToLowerCase, Success)
End Sub

private Sub mOpenSphericalCamera1_SessionUpdated(Success As Boolean, CameraResponse As JSONParser)
	Log("mOpenSphericalCamera1_SessionUpdated: "&Success)
End Sub

public Sub Release
	mMediaSession1.SetActive(False)
	mMediaSession1.Release
End Sub

public Sub RemoteEnable
	mMediaSession1.SetActive(True)
End Sub

public Sub RemoteDisable
	mMediaSession1.SetActive(False)
End Sub